import matplotlib as mpl
import numpy
import matplotlib.pyplot as plt
#import IPython
#written by Martin Sparre, copyright 2018


#we will work in natural units with G=1
G = 1.0

#Mass of the two bodies
M0 = 1.0
M1 = 0.3333

#Initial positions and velocities. x,y=0,0 is the center of mass the system.
#We set the velocity in the y-direction equal to the escape velocity.
#An offset in the x-direction is given, so the impact parameter is non-zero
X1,Y1,Z1 = 15.0,40.0,0.0
X0,Y0,Z0 = -M1/M0*X1,-M1/M0*Y1,0.0

Vesc = numpy.sqrt(2*G*M0/numpy.sqrt( (X0-X1)**2 + (Y0-Y1)**2 + (Z0-Z1)**2 ))
VX1,VY1,VZ1 = 0.0,-Vesc,0.0
VX0,VY0,VZ0 = 0.0,-M1/M0*VY1,0.0

#Keplers law say that tff = 0.5* pi R^1.5 / sqrt( 2 G (M0+M1), https://en.wikipedia.org/wiki/Free-fall_time
#so we will use this as an orbital timescale. Note that the formula is not exact because we have added an initial velocity and VX-value
OrbitalTimescale = 0.5*numpy.pi* numpy.sqrt( (X0-X1)**2 + (Y0-Y1)**2 + (Z0-Z1)**2 )**1.5 / numpy.sqrt(2*G*(M0+M1))

#Time integration parameters
t = 0.0#initial time
tmax = 1.0*OrbitalTimescale
dt = 0.000025*OrbitalTimescale
Leapfrog = True#if True Leapfrog is enabled, if False it is disabled

#Output times

OutputTimes = [0.31*tmax,0.4*tmax,0.50*tmax,0.55*tmax,0.6*tmax,0.625*tmax,0.65*tmax,0.675*tmax,0.9*tmax]

#Disk properties
N = 512
Ncircles = 5
Theta = numpy.linspace(0.0,Ncircles*2*numpy.pi, N,endpoint=False)
R = numpy.linspace(1,5.0,N)

x = R * numpy.cos(Theta) + X0
y = R * numpy.sin(Theta) + Y0
z = numpy.zeros(N) + Z0

vtheta = - numpy.sqrt(G*M0/R)#prograde merger
#vtheta = numpy.sqrt(G*M0/R)#retrograde

vr = 0.0
vx = vr * numpy.cos(Theta) - numpy.sin(Theta) * vtheta + VX0
vy = vr * numpy.sin(Theta) + numpy.cos(Theta) * vtheta  + VY0
vz = numpy.zeros(N) + VZ0

Output_i = 0

plt.figure(1,figsize=(4*len(OutputTimes),4))
plt.subplots_adjust(left=0.03,right=0.99, top=0.92,wspace=0.1, hspace=0.0,bottom=0.15)


#Create offset if Leapfrog method is enabled (if Leapfrog==False we will just use the Euler method)
if Leapfrog == True:
    ax = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (x-X0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (x-X1)
    ay = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (y-Y0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (y-Y1)
    az = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (z-Z0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (z-Z1)

    AX0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X0-X1)
    AY0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y0-Y1)
    AZ0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z0-Z1)

    AX1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X1-X0)
    AY1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y1-Y0)
    AZ1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z1-Z0)

    vx = vx - ax*dt/2.0
    vy = vy - ay*dt/2.0
    vz = vz - az*dt/2.0

    VX0 = VX0 - AX0*dt/2.0
    VY0 = VY0 - AY0*dt/2.0
    VZ0 = VZ0 - AZ0*dt/2.0

    VX1 = VX1 - AX1*dt/2.0
    VY1 = VY1 - AY1*dt/2.0
    VZ1 = VZ1 - AZ1*dt/2.0

#Main loop of integration
List_X0 = []
List_Y0 = []
List_X1 = []
List_Y1 = []
List_Time = []


while t<= tmax:
    #save coordinates to list so we can plot them later
    List_Time.append(t)
    List_X0.append(X0)
    List_Y0.append(Y0)
    List_X1.append(X1)
    List_Y1.append(Y1)

    #Calculate acceleration
    ax = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (x-X0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (x-X1)
    ay = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (y-Y0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (y-Y1)
    az = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (z-Z0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (z-Z1)

    AX0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X0-X1)
    AY0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y0-Y1)
    AZ0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z0-Z1)

    AX1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X1-X0)
    AY1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y1-Y0)
    AZ1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z1-Z0)

    #Advance velocity
    vx = vx + ax*dt
    vy = vy + ay*dt
    vz = vz + az*dt

    VX0 = VX0 + AX0*dt
    VY0 = VY0 + AY0*dt
    VZ0 = VZ0 + AZ0*dt

    VX1 = VX1 + AX1*dt
    VY1 = VY1 + AY1*dt
    VZ1 = VZ1 + AZ1*dt

    #Advance positions
    x = x + vx*dt
    y = y + vy*dt
    z = z + vz*dt

    X0 = X0+VX0*dt
    Y0 = Y0+VY0*dt
    Z0 = Z0+VZ0*dt

    X1 = X1+VX1*dt
    Y1 = Y1+VY1*dt
    Z1 = Z1+VZ1*dt

    t=t+dt

    if Output_i < len(OutputTimes):
        if t>OutputTimes[Output_i]:
            plt.subplot(1,len(OutputTimes),Output_i+1)
            plt.title(r"$t=%2.2f t_{\rm max}$"%(OutputTimes[Output_i]/tmax))
            plt.plot(X0,Y0,'o',mew=0,ms=8,color='orange',zorder=5)
            plt.plot(X1,Y1,'o',mew=0,ms=8,color='red',zorder=5)
            plt.plot(x,y,'o',mew=0,ms=2,color='blue',zorder=4)
            #do
            Output_i += 1
            plt.xlim((-50,50))
            plt.ylim((-50,50))


    if X1<0.0 and Y1>40.0:
        print 't/OrbitalTimescale:',t/OrbitalTimescale
        break
#plt.tight_layout()
#plt.show()
#convert lists to numpre arrays:
List_Time = numpy.array(List_Time)
List_X0 = numpy.array(List_X0)
List_Y0 = numpy.array(List_Y0)
List_X1 = numpy.array(List_X1)
List_Y1 = numpy.array(List_Y1)
Dist = numpy.sqrt((List_X0-List_X1)**2+(List_Y0-List_Y1)**2)


#Plot orbits
plt.figure(2)
plt.title('Orbit of the two galaxies')
ax = plt.subplot(1,1,1)
cmap_colors = plt.get_cmap('spectral')
ax.plot(List_X0, List_Y0,'-',lw=1,color='black',zorder=5)
ax.plot(List_X1, List_Y1,'-',lw=1,color='black',zorder=5)
ax.scatter(List_X0, List_Y0,c=List_Time/OrbitalTimescale,edgecolors='none',zorder=6)
ax.text(0.05, 0.95,'Minimum Distance = %2.2f'%Dist[Dist.argmin()], horizontalalignment='left',verticalalignment='center',transform=ax.transAxes,fontsize=16)
ax.text(0.05, 0.85,'tmin/torbit = %2.2f'%(List_Time[Dist.argmin()]/OrbitalTimescale), horizontalalignment='left',verticalalignment='center',transform=ax.transAxes,fontsize=16)

sc=plt.scatter(List_X1,List_Y1,c=List_Time/OrbitalTimescale,edgecolors='none',zorder=6,s=20)
color_bar = plt.colorbar(sc)
color_bar.set_label( r'$t/$OrbitalTimescale', color='black', fontsize=16)

plt.xlim((-50.0,50.0))
plt.ylim((-50.0,50.0))
plt.xlabel('$x$',fontsize=16)
plt.ylabel('$y$',fontsize=16)


plt.show()
